assert(LibStub, "Boing requires LibStub")
assert(LibStub:GetLibrary("CallbackHandler-1.0", true), "Boing requires CallbackHandler-1.0")
-----------------------
--  Local variables  --
-----------------------
addon = LibStub("AceAddon-3.0"):NewAddon("Boing", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0")
addon.L = LibStub:GetLibrary( "AceLocale-3.0" ):GetLocale(addon.GetName(addon))
addon.dataBroker = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject(addon.GetName(addon), {type = "data source", text = " ", icon = "Interface\\Icons\\Inv_misc_gear_03"})
---------------------------
--  Instantiate Objects  --
---------------------------

addon.confDefaults = {

}

function addon:OnInitialize()
    -- Code that you want to run when the addon is first loaded goes here.
    addon.db = LibStub("AceDB-3.0"):New(addon.GetName(addon) .. "DB", addon.confDefaults, true)
    addon.addonPath = "Interface\\Addons\\".. addon.GetName(addon) .."\\"
    addon.soundPath = "sounds\\"
    addon.charName = UnitName("player")
    addon.realmName = GetRealmName()
    addon:RegisterChatCommand(addon.GetName(addon):lower(), "PrintCharacterJumps")
    
    
    -- Verify session jumps variable
    if addon.db.char.session_jumps == nil then
        addon.db.char.session_jumps = 0
    end
    
    -- Verify per character jumps total variable
    if addon.db.char.jumps == nil then
        addon.db.char.jumps = 0
    end
    
    -- Verify per realm character jumps total variable
    if addon.db.realm.jumps == nil then
        addon.db.realm.jumps = {[addon.charName] = 0}
    end
    
    -- Verify global character jumps variable.
    if addon.db.global.jumps == nil then
        addon.db.global.jumps = {[addon.charName] = 0}
    end
    
    addon.db.char.session_jumps = 0
    addon.SetBrokerText()
end

----------------------
--  Hook Callbacks  --
----------------------
function addon:JumpOrAscendStart()
    -- Check to see if the player is falling
    -- Also useful to see if we've just jumped
    if IsFalling() then
        return
    -- Don't count movement while swimming
    elseif IsSwimming() then 
        return
    -- Don't count movement while on a flying mount
    elseif IsFlying() then
        return
    -- We're on a taxi, no jumping available!
    elseif UnitOnTaxi("player") then
        return
    -- User is mounted and standing still. Doesn't actually cause the mount to jump.
    elseif IsMounted() and not IsPlayerMoving() then
        return
    end
    PlaySoundFile(addon.addonPath .. addon.soundPath .. "JumpingSound.ogg")
    
    addon.db.char.session_jumps = addon.db.char.session_jumps + 1
    addon.SetBrokerText()
end

-----------------
--  Functions  --
-----------------

function addon:OnEnable()
    -- Called when the addon is enabled
    addon:SecureHook("JumpOrAscendStart")
    addon:RegisterEvent("PLAYER_LOGOUT")
    addon.db.char.session_jumps = 0
end

function addon:OnDisable()
    -- Called when the addon is disabled
    local totalJumps = addon.db.char.jumps + addon.db.char.session_jumps
    addon.db.char.jumps = totalJumps
    addon.db.realm.jumps[addon.charName] = totalJumps
    addon.db.global.jumps[addon.charName] = totalJumps
    addon:Unhook("JumpOrAscendStart")

end

function addon:PLAYER_LOGOUT()
    -- called when the player is logging out or reloading the screen.
    addon:Disable()
end

-- Prints the 
function addon:PrintCharacterJumps(input)
    local commands, jumps = input:lower(), addon:FormatJumps()

    if commands == "" then
        addon:Print(addon.L["CHARACTER_JUMPS_SESSION"](jumps[1]))
    elseif commands == "char" then
        addon:Print(addon.L["CHARACTER_JUMPS_TOTAL"](addon.charName, jumps[1], jumps[2]))
    elseif commands == "realm" then
        addon:Print(addon.L["REALM_JUMPS"](addon.realmName, jumps[3]))
    elseif commands == "all" then
        addon:Print(addon.L["GLOBAL_JUMPS"](jumps[4]))
    else
        addon:Print(addon.L["SLASH_HELP"](addon:GetName(addon)))
    end
end

function addon:FormatJumps()
    local session, character, realm, global =
        addon.comma_value(addon.db.char.session_jumps),
        addon.comma_value(addon.db.char.jumps + addon.db.char.session_jumps),
        addon.comma_value(addon.CalculateTotalRealmJumps(addon.db.realm.jumps)),
        addon.comma_value(addon.CalculateTotalGlobalJumps(addon.db.global.jumps))
        
    return {session, character, realm, global}
end

-- Calculates the total jumps across all characters on an account
function addon.CalculateTotalGlobalJumps()
    local globalJumps = 0
    
    for k, v in pairs(addon.db.global.jumps) do
        globalJumps = globalJumps + v
    end
    return globalJumps + addon.db.char.session_jumps
end

-- Calculates the total jumps across all characters on a realm
function addon.CalculateTotalRealmJumps()
    local realmJumps = 0
    
    for k, v in pairs(addon.db.realm.jumps) do
        realmJumps = realmJumps + v
    end
    return realmJumps + addon.db.char.session_jumps
end

-- add comma to separate thousands
-- Curtsey of http://lua-users.org/wiki/FormattingNumbers
-- credit http://richard.warburton.it
function addon.comma_value(n) 
    local left,num,right = string.match(n,'^([^%d]*%d)(%d*)(.-)$')
    return left..(num:reverse():gsub('(%d%d%d)','%1,'):reverse())..right
end

-- Update LDB Text
function addon.SetBrokerText()
    addon.dataBroker.text = addon.L["DATA_BROKER_JUMPS"](addon.db.char.session_jumps)
end

local function GetTipAnchor(frame)
	local x,y = frame:GetCenter()
	if not x or not y then return "TOPLEFT", "BOTTOMLEFT" end
	local hhalf = (x > UIParent:GetWidth()*2/3) and "RIGHT" or (x < UIParent:GetWidth()/3) and "LEFT" or ""
	local vhalf = (y > UIParent:GetHeight()/2) and "TOP" or "BOTTOM"
	return vhalf..hhalf, frame, (vhalf == "TOP" and "BOTTOM" or "TOP")..hhalf
end

function addon.dataBroker.OnEnter(self)
 	GameTooltip:SetOwner(self, "ANCHOR_NONE")
	GameTooltip:SetPoint(GetTipAnchor(self))
	addon.dataBroker.SetTooltipContents()
end

function addon.dataBroker.OnLeave()
	GameTooltip:Hide()
end

function addon.dataBroker.SetTooltipContents()
    local jumps = addon:FormatJumps()
	GameTooltip:ClearLines()
    GameTooltip:AddLine(addon.L['DATA_BROKER_TOOL_TITLE'])
    GameTooltip:AddLine(addon.L['DATA_BROKER_TOOL_CHARACTER_JUMPS'](addon.charName, jumps[2]))
    GameTooltip:AddLine(addon.L['DATA_BROKER_TOOL_REALM_JUMPS'](addon.realmName, jumps[3]))
    GameTooltip:AddLine(addon.L['DATA_BROKER_TOOL_GLOBAL_JUMPS'](jumps[4]))
	GameTooltip:Show()
end